/*
 * libarc.c -- libarc archive handler
 * (C)Copyright 1999,2000 by TAJIRI Yasuhiro
 * This file is part of Enfle.
 *
 * Last Modified: Mon Jun  5 03:40:07 2000.
 * $Id: libarc.c,v 1.2 2000/06/07 12:39:26 sian Exp $
 *
 * Enfle is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Enfle is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "enfle.h"
#include "mfile.h"
#include "libarc.h"
#include "utils.h"
#include "plugin.h"

#define MAX_DENSITY 5
#define INIT_TABLE_SIZE 1000

extern struct URL_module URL_module_file;
extern struct URL_module URL_module_http;
int libarc_archive_select(Archive *ar, int index);
int libarc_archive_read(Archive *ar, unsigned char *buf, int size);
int libarc_archive_close(Archive *ar);
int libarc_archive_seek(Archive *ar, long seek,int whence);
int libarc_archive_tell(Archive *ar);

int
#ifdef PIC
get_plugininfo(PluginInfo *p)
#else
archiver_libarc_get_plugininfo(PluginInfo *p)
#endif
{
  p->version = 1;
  p->type = _Archiver;
  p->pluginname = "Archiver Plugin via libarc version 0.2.1";
  p->pluginshortname = FORMAT_NAME;
  p->author = "TAJIRI Yasuhiro";
  p->dlhandle = NULL; /* set by plugin_load */
  p->functions.archiver.open = libarc_archive_open;
  return 1;
}

int
libarc_archive_open(Archive *ar)
{
    ArchiveFileList* archive_file_list = NULL;
    int nfiles = 1;

#ifdef DEBUG
    fprintf(stderr, "libarc_archive_open: open %s\n", ar->filename);
#endif

    url_add_modules(
	&URL_module_file,
	NULL);
    archive_file_list = make_archive_list(
        archive_file_list,1, &(ar->filename));
    if (archive_file_list != NULL) {
        char** new_files;
        int    new_nfiles;

        new_nfiles = nfiles;
        new_files = expand_archive_names(archive_file_list,
                                         &new_nfiles, &(ar->filename));
        if (new_files != NULL && archive_file_list != NULL) {
            LIBARC_info * info;

            if ((ar->info = calloc(1, sizeof(LIBARC_info))) == NULL) {
                fprintf(stderr, "No enough memory for info\n");
                goto error;
            }
            info = (LIBARC_info *)(ar->info);
            ar->nfiles=info->nfile  = new_nfiles;
            info->files  = new_files;
            info->url    = NULL;
            info->mem    = NULL;
            info->archive_file_list = archive_file_list;
        }
        else goto error;
    }
    else goto error;
    if (ar->nfiles) {
        ar->format = FORMAT_NAME;
        ar->select = libarc_archive_select;
        ar->close  = libarc_archive_close;
        ar->read   = libarc_archive_read;
        ar->seek   = libarc_archive_seek;
        ar->tell   = libarc_archive_tell;
        return 1;
    }
  error:
    if (ar->info) {
      free(ar->info);
      ar->info = NULL;
    }
    return 0;
}

int
libarc_archive_select(Archive *ar, int index)
{
    LIBARC_info *info = ar->info;

    if (ar == NULL)
        return 0;
    if (index < 0)
        return 0;
    if (info->url != NULL)
        url_close(info->url);
    if (info->mem == NULL)
      info->mem = mopen(10000);
    else
        mseek(info->mem, 0, SEEK_SET);
    info->url = archive_file_extract_open(info->archive_file_list,
                                          info->files[index]);
    if (info->url == NULL)
        info->url = url_open(info->files[index]);
    if (info->url == NULL) {
        fprintf(stderr, "Can't open: %s\n", info->files[index]);
        return 0;
    }
    ar->size = 0;
    
    while (1) {
        unsigned char buf[1000];
        int size;

	size = url_read(info->url, buf, sizeof(buf));
        if (size < 0) {
	  fprintf(stderr, "url_read returns %d: %d %s\n", size, url_errno, url_strerror(url_errno));
	  return 0;
	}
	ar->size += size;
        mwrite(buf, 1, size, info->mem);
        if(size != sizeof(buf))
            break;
    }
    mseek(info->mem, 0, SEEK_SET);
    return 1;
}

int
libarc_archive_seek(Archive *ar, long seek,int whence)
{
    return mseek(((LIBARC_info*)(ar->info))->mem, seek, whence);
}

int
libarc_archive_tell(Archive *ar)
{
    return mtell(((LIBARC_info*)(ar->info))->mem);
}
        

int
libarc_archive_read(Archive *ar, unsigned char *buf, int size)
{
    return mread(buf, 1, size, ((LIBARC_info*)(ar->info))->mem);
}
        
int
libarc_archive_close(Archive *ar)
{
    mclose(((LIBARC_info*)(ar->info))->mem);
    free(ar->info);
    ar->info = NULL;
    return 1;
}
